#include <math.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>

#include <X11/Xlib.h>
#include <X11/keysym.h>

#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/PushB.h>

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/GLwMDrawA.h>

#include "estrut.h"

#define TOTAL_POINTS 400
#define DISPLAY_LIST 1


/* Global rotation angles and matrix */
GLfloat anglex, angley;
GLfloat rot_matrix[16];

int converte_esf_cart(double, double, double, double *,
                      double *, double *);

void point_image();
void motif(int);
void modl3d(int);


static void InitDisplay (void)
{
  int i,j;
  double x, y, z;
  struct imag_cart *f;
  struct imag_esf *g;
  extern int X_MAX, Y_MAX;
  extern double raio;

  // Set clear color;
  // glClearColor(0.5, 0.5, 0.5, 0.0);
  glClearColor(0.2, 0.2, 0.2, 0.0);

  // Create  points;
  glColor3f(1.0, 1.0, 1.0);
  glNewList(DISPLAY_LIST, GL_COMPILE);
  glBegin(GL_POINTS);
  for (i=0; i<X_MAX; i++) {
    for (j=0; j<Y_MAX; j++){
      for (f = entry_array[i][j]; f; f=f->prox)
      {
	/*
       	
	if ( f->close_face )  glColor3f(1.0, 0.3, 0.3);
	else glColor3f(1.0, 1.0, 1.0);
      
	if ( f->pto_mdl )  glColor3f(0.0, 0.0, 1.0);
	if ( f->new_imag ) glColor3f(0.0, 1.0, 0.0);
	*/
	x=f->X.x/raio; y=f->X.y/raio,z=f->X.z/raio;
	/*
  for (i=0; i<T_MAX; i++) {
    for (j=0; j<P_MAX; j++){
      for (g = entry_array_esf[i][j]; g; g=g->prox){
	converte_esf_cart(g->rho,g->theta,g->phi,&x,&y,&z);
	x=x/raio; y=y/raio,z=z/raio;
  */
	glVertex3d(x,y,z);
      }
    }
  }
  glEnd();
  glEndList();

  /* Enable depth testing */
  glEnable(GL_DEPTH_TEST);

  /* Enable and define fog */
  glEnable(GL_FOG);
  glFogf(GL_FOG_MODE, GL_LINEAR);
  glFogf(GL_FOG_START, 3.0);
  glFogf(GL_FOG_END, 6.0);

  /* Define projection matrix */
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(45.0, 1.0, 0.5, 10.0);

  /* Define model viewing matrix */
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glGetFloatv(GL_MODELVIEW_MATRIX, rot_matrix);
  glTranslatef(0.0, 0.0, -4.0);

  /* Initialize rotation angles */
  anglex=0.0;
  angley=0.0;
}


void DrawDisplay (Widget w)
{
  /* Clear frame and z buffers */
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  /* Call previously compiled list */
  glCallList(DISPLAY_LIST);

  /* Sync graphic drawing requests and swap buffers */
  GLwDrawingAreaSwapBuffers(w);
}


void ExposeCallback (Widget w, XtPointer client_data, XtPointer call_data)
{
  DrawDisplay(w);
}


void EventHandler (Widget w, XtPointer client_data, XEvent *event,
                   Boolean *cont)
{
  KeySym ks;
  static int lastx, lasty;
  int redraw, recalc;
  
  redraw=recalc=0;
  switch (event->type) {
    case KeyPress:
      /* Find which key was pressed */
      ks=XLookupKeysym((XKeyEvent *)event, 0);
      switch (ks) {
        case 'd':
        case 'D':
          if (glIsEnabled(GL_DEPTH_TEST))
            glDisable(GL_DEPTH_TEST);
          else
            glEnable(GL_DEPTH_TEST);
          redraw=1;
          break;
        case 'f':
        case 'F':
          if (glIsEnabled(GL_FOG))
            glDisable(GL_FOG);
          else
            glEnable(GL_FOG);
          redraw=1;
          break;
        case '1':
        case '2':
        case '3':
        case '4':
          glPointSize(1.0+ks-'1');
          redraw=1;
          break;
      }
      break;
      
    case MotionNotify:
      /* Check for mouse motion with button#1 pressed */
      if (event->xmotion.state == Button1Mask) {
        angley=0.6*(event->xmotion.x-lastx);
        anglex=0.6*(event->xmotion.y-lasty);
        lastx=event->xmotion.x;
        lasty=event->xmotion.y;
        recalc=1;
      }
      break;
      
    case ButtonPress:
      /* Check if button#1 was pressed */
      if (event->xbutton.button == Button1) {
        /* Set input focus to this widget */
        XmProcessTraversal(w, XmTRAVERSE_CURRENT);
        lastx=event->xbutton.x;
        lasty=event->xbutton.y;
      }
      break;
  }

  /* Recalc the model viewing matrix if needed */
  if (recalc) {
    /* First, apply rotations to our private rotation
       matrix and store it */
    glLoadIdentity();
    glRotatef(anglex, 1.0, 0.0, 0.0);
    glRotatef(angley, 0.0, 1.0, 0.0);
    glMultMatrixf(rot_matrix);
    glGetFloatv(GL_MODELVIEW_MATRIX, rot_matrix);

    /* Then, use the previously updated matrix to
       build the model view matrix */
    glLoadIdentity();
    glTranslatef(0.0, 0.0, -4.0);
    glMultMatrixf(rot_matrix);
    redraw=1;
  }

  /* Redisplay widget if needed */
  if (redraw)
    DrawDisplay(w);
}


void ResizeCallback (Widget w, XtPointer client_data, XtPointer call_data)
{
  GLwDrawingAreaCallbackStruct *data;

  data=(GLwDrawingAreaCallbackStruct *)call_data;

  /* Change projection matrix to maintain aspect ratio */
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(45.0, (GLdouble)data->width/data->height, 0.5, 100.0);
  glMatrixMode(GL_MODELVIEW);
  
  /* Update viewport dimensions */
  glViewport(0, 0, data->width, data->height);
}


void QuitCallback (Widget w, XtPointer client_data, XtPointer call_data)
{
  exit(0);
}


void point_image()
{
    int argc = 0;
  char **argv;
  GLXContext glx_context;  
  Arg args[32];
  Widget top, frame, form, quit, glxarea;
  XtAppContext app_context;
  
  Colormap cmap;
  Display *dpy;
  XVisualInfo *vi;
  
  static int snglBuf[]={GLX_RGBA, GLX_DEPTH_SIZE, 16, None};
  static int dblBuf[]={GLX_RGBA, GLX_DEPTH_SIZE, 16, GLX_DOUBLEBUFFER, None};
  int install_cmap, n;

  /* Create top level widget */
  n=0;
  XtSetArg(args[n], XmNtitle, "points"); n++;
  top=XtAppInitialize(&app_context, "points", NULL, 0, (Cardinal *)&argc,
                      argv, NULL, args, n);
  dpy=XtDisplay(top);
  
  /* Test command line */
  install_cmap=0;
  if (argc==2 && !strcmp(argv[1], "-install"))
    install_cmap=1;

  /* Find an OpenGL-capable RGB visual with depth buffer */
  vi=glXChooseVisual(dpy, DefaultScreen(dpy), dblBuf);
  if (vi == NULL) {
    vi=glXChooseVisual(dpy, DefaultScreen(dpy), snglBuf);
    if (vi == NULL)
      XtAppError(app_context, "no RGB visual with depth buffer");
  }
  
  /* Create a visual context */
  glx_context=glXCreateContext(dpy, vi,
                               /* no display list sharing */ None,
                               /* no direct rendering */ GL_FALSE);
  if (glx_context == NULL) 
    XtAppError(app_context, "could not create rendering context");

  /* Create an X colormap if not using the default visual */
  if (!install_cmap && vi->visualid ==
      XVisualIDFromVisual(DefaultVisual(dpy, DefaultScreen(dpy))))
    cmap=DefaultColormap(dpy, DefaultScreen(dpy));
  else
    cmap=XCreateColormap(dpy, RootWindow(dpy, vi->screen), 
                         vi->visual, AllocNone);

  /* Set the XmNdepth, XmNcolormap and XmNvisual  in top widget */ 
  n=0;
  XtSetArg(args[n], XmNdepth, vi->depth); n++;
  XtSetArg(args[n], XmNvisual, vi->visual); n++;
  XtSetArg(args[n], XmNcolormap, cmap); n++;
  XtSetValues(top, args, n);

  /* Create form widget to manage the placement of children */
  form=XmCreateForm(top, "form", NULL, 0);
  XtManageChild(form);

  /* Create a push button: add the callback procedure after managing */
  n=0;
  XtSetArg(args[n], XmNlabelString, XmStringCreateSimple("Quit!")); n++;
  XtSetArg(args[n], XmNborderWidth, 2); n++;
  XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
  XtSetArg(args[n], XmNtopOffset, 10); n++;
  XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
  XtSetArg(args[n], XmNrightOffset, 10); n++;
  XtSetArg(args[n], XmNwidth, 70); n++;
  XtSetArg(args[n], XmNheight, 40); n++;
  quit=XmCreatePushButton(form, "quit", args, n);
  XtManageChild(quit);
  XtAddCallback(quit, XmNactivateCallback, QuitCallback, NULL);
  
  /* Create frame widget to outline the drawing area */
  n=0;
  XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
  XtSetArg(args[n], XmNtopOffset, 10); n++;
  XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
  XtSetArg(args[n], XmNbottomOffset, 10); n++;
  XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
  XtSetArg(args[n], XmNleftOffset, 10); n++;
  XtSetArg(args[n], XmNrightAttachment, XmATTACH_WIDGET); n++;
  XtSetArg(args[n], XmNrightWidget, quit); n++;
  XtSetArg(args[n], XmNrightOffset, 10); n++;
  frame=XmCreateFrame(form, "frame", args, n);
  XtManageChild(frame);

  /* Create a drawing area with the same visual as the toplevel widget */
  n=0;
  XtSetArg(args[n], GLwNrgba, TRUE); n++;
  XtSetArg(args[n], GLwNvisualInfo, vi); n++;
  XtSetArg(args[n], GLwNinstallColormap, FALSE); n++;
  XtSetArg(args[n], XmNtraversalOn, TRUE); n++; /* allow keyboard traversal */
  XtSetArg(args[n], XmNdepth, vi->depth); n++;
  XtSetArg(args[n], XmNwidth, 500); n++;
  XtSetArg(args[n], XmNheight, 500); n++;
  glxarea=XtCreateWidget("glxarea", glwMDrawingAreaWidgetClass,
                         frame, args, n);
  XtManageChild(glxarea);
  XtAddCallback(glxarea, GLwNexposeCallback, ExposeCallback, NULL);
  XtAddCallback(glxarea, GLwNresizeCallback, ResizeCallback, NULL);
  XtAddEventHandler(glxarea,
                    ButtonPressMask | Button1MotionMask | KeyPressMask,
                    False, EventHandler, NULL);

  /* Set colormap for drawing area */
  n=0;
  XtSetArg(args[n], XmNcolormap, cmap); n++;
  XtSetValues(glxarea, args, n);

  /* Realize shell widget */
  XtRealizeWidget(top);

  /* Once widget is realized, bind the rendering context to the windows */
  GLwDrawingAreaMakeCurrent(glxarea, glx_context);

  InitDisplay();
  
  XtAppMainLoop(app_context);
}


void motif(int Obj)
{
return;
}
void modl3d(int Obj)
{
return;
}
